require "optparse"

module GithubMembers
  class Options
    class ParseError < Error; end

    attr_accessor :github_org
    attr_accessor :github_token
    attr_accessor :markdown_file
    attr_accessor :yaml_file
    attr_accessor :fields
    attr_accessor :help
    attr_accessor :help_shown
    attr_accessor :version

    def initialize(argv)
      self.github_token = ENV["GITHUB_TOKEN"].to_s

      args = parser.parse!(argv)
      self.help = parser.help + help_examples(parser.program_name)
      self.github_org = args.first
      self.markdown_file = Pathname(markdown_file || Defaults::MARKDOWN_FILE)
      self.yaml_file = Pathname(yaml_file || Defaults::YAML_FILE)
    rescue OptionParser::ParseError => e
      raise ParseError, e.message
    end

    private def parser
      @parser ||= OptionParser.new do |opts|
        opts.banner = "Usage: #{opts.program_name} [options] <org>"

        opts.on("-g", "--github-token GITHUB_TOKEN",
          "A GitHub access token. The `GITHUB_TOKEN` environment variable",
          "is also available.") do |v|
          self.github_token = v
        end

        opts.on("-m", "--markdown-file [FILE]", "A Markdown file path. Default: `#{Defaults::MARKDOWN_FILE}`") do |v|
          self.markdown_file = v
        end

        opts.on("-y", "--yaml-file [FILE]", "A YAML file path. Default: `#{Defaults::YAML_FILE}`") do |v|
          self.yaml_file = v
        end

        opts.on("-f", "--fields [FIELDS]", Array, "A comma-separated list of a member additional fields") do |v|
          self.fields = v
        end

        opts.on_tail("-h", "--help", "Show help") do
          self.help_shown = true
        end

        opts.on_tail("-v", "--version", "Show version") do
          self.version = VERSION
        end
      end
    end

    private def help_examples(program_name)
      <<~HELP

        Examples:
            # By default
            #{program_name} <org>

            # Set a token
            GITHUB_TOKEN=*** #{program_name} <org>
            #{program_name} <org> -g ***

            # Specify files
            #{program_name} <org> -m README.md -y company_members.yml

            # Add fields
            #{program_name} <org> -f slack,note
      HELP
    end
  end
end
